#!/usr/bin/perl
#
# $Id: fetchem.pl,v 1.10 1998/07/14 17:59:59 wralph Exp $
#
#  Script to fetch tcpdump gzipped hourly logfiles from a sensor,
#  move them to an dated subdirectory on the analyzer, run through
#  a filter looking for suspicious events, and position that suspicious
#  events text file on a web page.
#  
#  Written by Bill Ralph 3/12/98
#
# Set up some variables.
#
use Getopt::Long;
use POSIX qw(strftime);
use Time::Local;
#
$CID_PATH = "/usr/local/logger";
$ENV{PATH} = "/bin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:$CID_PATH";
#
sub usage {
        print "Usage: fetchem -l Site [-debug] [-d SNIFDATE].\n";
	exit 2;
}
#
#	Parse the parameters.
#
#
&GetOptions("debug", \$debug, "d:s", \$snifdate, "l=s", \$Site);

##
#
#  Rid ourselves of standard output and error to prevent tons of mail messages
#  from cron.
#
@lvls = split(/\//,$0);				# Split path
($call, $suf) = split(/\./, $lvls[$#lvls]);	# Split call.suffix
$LOG_FILE = $debug ? "/tmp/$call.log" : "/dev/null";
open(STDOUT, ">>$LOG_FILE");
select STDOUT; $| = 1;
open(STDERR, ">&STDOUT");
select STDERR; $| = 1;              # make unbuffered

#
# Writer a marker to the log file.
#
$marker=strftime("%c", localtime(time));
print STDOUT $marker;
foreach $param ($0, @ARGV) {
   printf " %s", $param;
}
print STDOUT "\n";
#
#  Check parameter validity.
#
if ("$Site" eq "")
{
	usage();
}

#
# Once the Site is identified from the command line,
# load the needed external Parameters.
#
unshift(@INC, "$CID_PATH/sites");
require "${Site}.ph";

#
#
# If we have been called with a SNIFDATE parameter, set up the $subdir variable,
# else download the SNIFDATE from the sensor.
#
if ($snifdate eq "") {
   $snif_file="/tmp/" . $SITE . "_snif.date";

   print STDOUT "SNIF file = $snif_file \n";

   if ( -f $snif_file) {
      unlink $snif_file;
   }

   print STDOUT ("scp root\@${SENSOR}:$SENSOR_DIR/snif.date.0 $snif_file\n");

   system("scp root\@${SENSOR}:$SENSOR_DIR/snif.date.0 $snif_file") == 0 or
     die("Unable to connect to $SENSOR.");
   $snifdate=`cat $snif_file`;
   chomp($snifdate);
}
#
#  Break up the "snifdate" into its useful components.
#
$year=substr($snifdate,0,2);
$mon=substr($snifdate,2,2) - 1;
$mday=substr($snifdate,4,2);
$sn1=substr($snifdate,0,-2);
$hour=substr($snifdate,-2,2).":00";
$hour_only = substr($hour,0,2);
$subdir=strftime("%h%d", 0, 0, 0, $mday, $mon, $year, 0, 0, 0);
#
# Convert our snif date back to time format.
#
$snif_time = timelocal(0, 0, $hour_only, $mday, $mon, $year);
@yester_time = localtime($snif_time - 86400);
@tomorrow_time = localtime($snif_time + 86400);
$yester_dir = strftime("%h%d", @yester_time);
$yester_file = "tcp." . strftime("%y%m%d", @yester_time) . "00.html";
$tomorrow_dir = strftime("%h%d", @tomorrow_time);
$tomorrow_file = "tcp." . strftime("%y%m%d", @tomorrow_time) . "00.html";

print STDOUT "snifdate = $snifdate, sn1 = $sn1, dir = $subdir, hour = $hour\n";
print STDOUT "yester_file = $yester_file, tommorow_file = $tomorrow_file\n";

#
# Predict the previous hour and next hour for html links.
#
$last_hour = strftime("%y%m%d%H", localtime($snif_time - 3600));
$last_hour_dir = "../".strftime("%h%d", localtime($snif_time - 3600));
#
$next_hour = strftime("%y%m%d%H", localtime($snif_time + 3600));
$next_hour_dir = "../".strftime("%h%d", localtime($snif_time + 3600));

print STDOUT "Last HR. = $last_hour, Next HR. = $next_hour\n";
print STDOUT "Last HR/dir = $last_hour_dir, Next HR/dir = $next_hour_dir\n";


#
#  Make sure subdirectory "MONXX" exists under $ANALYZER_DIR and
#  on web page
#
unless ( -d "$ANALYZER_DIR/$subdir") {
   mkdir("$ANALYZER_DIR/$subdir", 0755) 
    or die "Unable to mkdir $ANALYZER_DIR/$subdir: $?";
}
unless ( -d "${OUTPUT_WEB_DIR}/$subdir") {
   mkdir("${OUTPUT_WEB_DIR}/$subdir", 0755) 
    or die "Unable to mkdir ${OUTPUT_WEB_DIR}/$subdir";
}

#
#  Create the index for the day represented by the directory we just 
#  created. The index.html files will be a page listing one html file for
#  each hour of the day - prettified a tiny bit.
#
chdir("/${OUTPUT_WEB_DIR}/$subdir");
   unless ( -e "index.html") {
   open(DAYINDEX, ">index.html");
   print DAYINDEX <<"EOF";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Final//EN">
<HTML>
<HEAD>
  <TITLE>SHADOW results page for Site: $SITE on $subdir</TITLE>
 </HEAD>
<FRAMESET ROWS="95,*" BORDER=10 FRAMEBORDER=7>
<FRAME SRC="navbar.html" NAME="navbar" SCROLLING=NO NORESIZE>
<FRAME SRC="${sn1}00.html" NAME="data" SCROLLING=YES>
</FRAMESET>
</HTML>
EOF
   close(DAYINDEX);
   open(NAV_HTML, ">navbar.html");
   print NAV_HTML <<"EOF";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Final//EN">
<HTML>
<BODY BGCOLOR="#bebebe">
<TABLE ALIGN=LEFT BORDER=5 BORDERCOLOR="#0000" CELLSPACING="5" CELLPADDING="5">
<TR>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="../$yester_dir/index.html" TARGET="_top"><B>Yesterday</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="/cgi-bin/dir-it.cgi" TARGET="_top"><B>Sites</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="../$tomorrow_dir/index.html" TARGET="_top"><B>Tomorrow</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="/tcpdump_results/index.html" TARGET="_top"><B>Home</B></A></TD></TR>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="/cgi-bin/pat_match_form.cgi" TARGET="_blank"><B>Search</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="/cgi-bin/scan_search.cgi" TARGET="_blank"><B>SS</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="/cgi-bin/lookup.cgi" TARGET="_blank"><B>Lookup</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="/cgi-bin/whois.cgi" TARGET="_blank"><B>Whois</B></A></TD></TR>
</TABLE>
<TABLE ALIGN=CENTER BORDER=5 BORDERCOLOR="#0000" CELLSPACING="5" CELLPADDING="5">
<TR>
<TD ALIGN=CENTER VALIGN=MIDDLE><A><B>AM</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}00.html" TARGET="data"><B>00</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}01.html" TARGET="data"><B>01</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}02.html" TARGET="data"><B>02</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}03.html" TARGET="data"><B>03</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}04.html" TARGET="data"><B>04</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}05.html" TARGET="data"><B>05</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}06.html" TARGET="data"><B>06</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}07.html" TARGET="data"><B>07</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}08.html" TARGET="data"><B>08</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}09.html" TARGET="data"><B>09</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}10.html" TARGET="data"><B>10</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}11.html" TARGET="data"><B>11</B></A></TD></TR>
<TR>
<TD ALIGN=CENTER VALIGN=MIDDLE><A><B>PM</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}12.html" TARGET="data"><B>12</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}13.html" TARGET="data"><B>13</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}14.html" TARGET="data"><B>14</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}15.html" TARGET="data"><B>15</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}16.html" TARGET="data"><B>16</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}17.html" TARGET="data"><B>17</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}18.html" TARGET="data"><B>18</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}19.html" TARGET="data"><B>19</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}20.html" TARGET="data"><B>20</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}21.html" TARGET="data"><B>21</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}22.html" TARGET="data"><B>22</B></A></TD>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="${sn1}23.html" TARGET="data"><B>23</B></A></TD></TR>
</TABLE>
</BODY>
</HTML>
EOF
   close(NAV_HTML);
}
#
# Copy down the raw gzipped tcpdump data file.
#
unless ( -f "$ANALYZER_DIR/$subdir/tcp.$snifdate.gz" ) {

      print STDOUT ("scp root\@${SENSOR}:$SENSOR_DIR/tcp.$snifdate.gz $ANALYZER_DIR/$subdir\n");

      system("scp root\@${SENSOR}:$SENSOR_DIR/tcp.$snifdate.gz $ANALYZER_DIR/$subdir") == 0 or
         die("Unable to copy zipped Data file from $SENSOR.");
}
#
#  Call tcpdump to read the newly fetched file, scan it with our bad guys
#  filters, and write the results to the web page.
#
$zipped_file="$ANALYZER_DIR/$subdir/tcp.$snifdate.gz";
chdir($FILTER_DIR);
@filters = glob("*.filter");

   print STDOUT "Filter names: @filters\n";

$output_txt_file="$OUTPUT_WEB_DIR/$subdir/$snifdate.txt";
$output_html_file="$OUTPUT_WEB_DIR/$subdir/$snifdate.html";
$output_index_file="$OUTPUT_WEB_DIR/$subdir/index.html";
$prev_out_file="$last_hour_dir/$last_hour.html";
$next_out_file="$next_hour_dir/$next_hour.html";

   print STDOUT "zip file = $zipped_file\n";
   print STDOUT "output(txt) = $output_txt_file\nprev = $prev_out_file\n";
   print STDOUT "output(html) = $output_html_file\nnext = $next_out_file\n";

open(OUTPUT, ">$output_html_file");
print OUTPUT <<"EOF";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Final//EN">
<HTML>
<HEAD>
<TITLE>Hourly tcpdump for $SITE on $subdir at $hour.</TITLE>
</HEAD>
<BODY BGCOLOR='ffdead'> 
<H3>Site: $SITE - Date: $subdir.</H3>
<PRE>
EOF
close(OUTPUT);
if (not -e $output_txt_file) {
   system("touch $output_txt_file");
#
#  Loop through site filters, call tcpdump and save the output text file.
#
   for $fil (@filters) {

      print STDOUT ("gunzip -c $zipped_file | tcpdump -n -r - -F $fil >> $output_file\n");

      system("gunzip -c $zipped_file | tcpdump -n -r - -F $fil >> $output_txt_file");
   }
}

#
# Clean  some tcpdump crap from the file before passing it along.
#
print STDOUT "Cleaning ${output_txt_file}.\n";

open(CLEAN, ">${output_txt_file}.cleaned") or 
        die "Can't open ${output_txt_file}.cleaned";
open(TEXTFILE, "<${output_txt_file}") or die "Can't open ${output_txt_file}";
while (<TEXTFILE>) {
   next if /gre-proto/;
   next if /trunc/;
   print CLEAN $_;
}
close(CLEAN);
close(TEXTFILE);
unlink("${output_txt_file}");
rename("${output_txt_file}.cleaned", "${output_txt_file}");
print STDOUT "Text file cleaned of tcpdump exceptions.\n";
#
print STDOUT "Calling sort_and_resolve.pl\n";

system("perl $CID_PATH/sort_and_resolve.pl $output_txt_file");

print STDOUT "Copying ${output_txt_file}.sorted to $output_html_file.\n";

open(OUTPUT, ">>$output_html_file") or die "Can't open $output_html_file";
open(TEXTFILE, "<${output_txt_file}.sorted") or die "Can't open ${output_txt_file}.sorted";
while (<TEXTFILE>) {
   print OUTPUT $_;
}
close(TEXTFILE);
print OUTPUT <<"EOF";
<HR SIZE=3>
EOF
#
# Run script to track those source IP addresses that hit multiple destination
# addresses (greater than a threshhold).
#
print STDOUT "Calling $CID_PATH/find_scan.pl $SITE $zipped_file.\n";

system("perl $CID_PATH/find_scan.pl $SITE $zipped_file");

print STDOUT "Copying results to html file\n";

open(RESULTS, "</tmp/${SITE}.results") 
   or die "Can't open /tmp/${SITE}.results";
while (<RESULTS>) {
   print OUTPUT $_;
}
close(RESULTS);
print OUTPUT <<"EOF";
</PRE>
<H3>Site: $SITE - Date: $subdir.</H3>
<HR>
<TABLE CELLSPACING="0" CELLPADDING="0">
<TR>
<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="$prev_out_file"><IMG SRC="/images/navbars/2/1.gif" WIDTH="90" HEIGHT="20" BORDER="0" HSPACE="0" VSPACE="0"></A><IMG SRC="/images/navbars/2/2.gif" WIDTH="110" HEIGHT="20" HSPACE="0" VSPACE="0"><A HREF="/cgi-bin/dir-it.cgi" TARGET="_top"><IMG SRC="/images/navbars/2/3.gif" WIDTH="50" HEIGHT="20" BORDER="0" HSPACE="0" VSPACE="0"></A><IMG SRC="/images/navbars/2/2.gif" WIDTH="110" HEIGHT="20" HSPACE="0" VSPACE="0"><A HREF="$next_out_file"><IMG SRC="/images/navbars/2/5.gif" WIDTH="90" HEIGHT="20" BORDER="0" HSPACE="0" VSPACE="0"></A></TD></TR>
</TABLE>

</BODY>
</HTML>
EOF
close(OUTPUT);
#
unlink("${output_txt_file}.sorted")
   or die("Unable to remove ${output_txt_file}.sorted.");
unlink("/tmp/${SITE}.results")
   or die("Unable to remove /tmp/${SITE}.results.");
print STDOUT "Fetchem completed.\n";
